import sys

import pytest


def inc(x):
    return x + 1


@pytest.fixture
def print_extra_msg():
    print("this is setup")
    yield
    print("this is teardown")


@pytest.mark.high
@pytest.mark.owner("foo")
@pytest.mark.extra_attributes({"env": ["AA", "BB"]})
def test_success(print_extra_msg):
    """
    测试获取答案
    """
    print("this is print sample output")
    assert inc(3) == 4


def test_failed():
    print("this is assert output")
    assert inc(3) == 6


@pytest.mark.low
@pytest.mark.owner("bar")
def test_raise_error():
    print("this is raise output", file=sys.stderr)
    assert inc(3) == 4
    raise RuntimeError("this is raise runtime error")



import json
import logging
import os
import shutil
import sys
import tempfile
from dataclasses import asdict
from pathlib import Path
from unittest.mock import MagicMock
from xml.dom import minidom

import pytest

from src.testsolar_pytestx.extend.coverage_extend import (
    ProjectPath,
    TestFileLines,
    TestCaseCoverage,
    Coverage,
    CoverageData,
    check_coverage_enable,
    collect_code_packages,
    filter_coverage_xml_packages,
    convert_coverage_data,
    prepare_file_path,
    get_testcase_coverage_data,
    find_coverage_db_path,
    generate_coverage_json_file,
    collect_coverage_report,
)

testdata_dir = Path(__file__).parent.parent.absolute().joinpath("testdata/testsolar_coverage")
logger = logging.getLogger(__name__)


def test_coverage_to_json():
    # 创建 ProjectPath 对象
    project_path = ProjectPath(
        projectPath="/path/to/project",
        beforeMove="/path/to/project/before",
        afterMove="/path/to/project/after",
    )

    # 创建 TestFileLines 对象
    test_file_lines1 = TestFileLines(fileName="file1.py", fileLines=[1, 2, 3])
    test_file_lines2 = TestFileLines(fileName="file2.py", fileLines=[4, 5, 6])

    # 创建 TestCaseCoverage 对象
    test_case_coverage1 = TestCaseCoverage(caseName="test_case_1", testFiles=[test_file_lines1])
    test_case_coverage2 = TestCaseCoverage(caseName="test_case_2", testFiles=[test_file_lines2])

    # 创建 Coverage 对象
    coverage = Coverage(
        coverageFile="/path/to/coverage.xml",
        coverageType="cobertura_xml",
        projectPath=project_path,
        caseCoverage=[test_case_coverage1, test_case_coverage2],
    )

    # 将 Coverage 对象序列化为 JSON
    json_output = coverage.to_json()
    expected_output = json.dumps(asdict(coverage), indent=4)

    # 验证 JSON 输出
    assert json.loads(json_output) == json.loads(expected_output)


def test_check_coverage_enable_true(monkeypatch):
    assert check_coverage_enable() is False

    monkeypatch.setenv("TESTSOLAR_TTP_ENABLECOVERAGE", "1")
    assert check_coverage_enable() is True

    monkeypatch.setenv("TESTSOLAR_TTP_ENABLECOVERAGE", "true")
    assert check_coverage_enable() is True


def test_collect_code_packages(monkeypatch):
    """
    测试 collect_code_packages 函数
    """
    # 创建临时文件目录并扫描
    temp_dir = Path(os.getcwd()) / ".test_temp_dir"
    if temp_dir.exists():
        shutil.rmtree(temp_dir)
    temp_dir.mkdir()
    (temp_dir / "__init__.py").touch()
    result = collect_code_packages([])
    assert sorted(result) == sorted(["tests", "src"])
    shutil.rmtree(temp_dir)

    monkeypatch.setenv("TESTSOLAR_TTP_COVERAGECODEPACKAGES", "")
    assert collect_code_packages(["src/tests/test_add.py"]) == ["tests"]

    # 使用 monkeypatch 确保环境变量在测试结束后恢复
    monkeypatch.delenv("TESTSOLAR_TTP_COVERAGECODEPACKAGES", raising=False)

    # 测试用例列表为空，则返回当前目录的包名
    result = collect_code_packages([])
    assert sorted(result) == sorted(["tests", "src"])

    # 设置环境变量 TESTSOLAR_TTP_COVERAGECODEPACKAGES
    monkeypatch.setenv("TESTSOLAR_TTP_COVERAGECODEPACKAGES", "src")
    result = collect_code_packages([])
    assert result == ["src"]

    # 测试用例列表中包含文件路径
    result = collect_code_packages(["src/tests/test_add.py"])
    assert result == ["src"]


@pytest.fixture
def coverage_file_path(tmpdir):
    # 假设你的测试数据目录是 testdata_dir
    original_coverage_file = testdata_dir / "coverage.xml"

    # 将原始文件复制到临时目录中
    temp_coverage_file = Path(tmpdir) / "coverage.xml"
    shutil.copy(original_coverage_file, temp_coverage_file)

    return temp_coverage_file


def test_filter_coverage_xml_packages(monkeypatch, coverage_file_path):
    # 捕获日志输出
    log_output = []

    def mock_log_info(message):
        log_output.append(message)

    monkeypatch.setattr(logger, "info", mock_log_info)

    # 调用函数
    filter_coverage_xml_packages(coverage_file_path, ["package1", "package2"])

    # 验证修改后的文件内容
    with open(coverage_file_path, "r") as f:
        dom = minidom.parse(f)
        packages = dom.getElementsByTagName("package")
        package_names = [package.getAttribute("name") for package in packages]
        assert "package1" in package_names
        assert "package2" in package_names
        assert "package3" not in package_names


def test_convert_coverage_data():
    # 准备输入数据
    result = {"test_case1": CoverageData(name="test_case1", files={"file1.py": [1, 2]})}
    rav_fn = "file2.py"
    line_map = [3, 4, 5, 6]
    context_map = {
        3: ["test_case2|context"],
        4: ["test_case2|context", "test_case3|context"],
        5: ["test_case3|context"],
        6: ["test_case4"],  # 触发 ValueError
    }

    # 调用方法
    updated_result = convert_coverage_data(result, rav_fn, line_map, context_map)

    # 验证结果
    expected_result = {
        "test_case1": CoverageData(name="test_case1", files={"file1.py": [1, 2]}),
        "test_case2": CoverageData(name="test_case2", files={"file2.py": [3, 4]}),
        "test_case3": CoverageData(name="test_case3", files={"file2.py": [4, 5]}),
        "test_case4": CoverageData(name="test_case4", files={"file2.py": [6]}),
    }

    assert updated_result == expected_result


def test_convert_coverage_data_path_handling():
    # 准备输入数据
    result = {}
    rav_fn = "path/to/file2.py"
    line_map = [7]
    context_map = {7: ["path.to.test_case5|context"]}

    # 调用方法
    updated_result = convert_coverage_data(result, rav_fn, line_map, context_map)

    # 验证结果
    expected_result = {
        "path.to.test_case5": CoverageData(
            name="path.to.test_case5", files={"path/to/file2.py": [7]}
        )
    }

    assert updated_result == expected_result


def test_prepare_file_path():
    # 测试用例1：文件路径以根路径开头
    fn = "/root/path/to/file.py"
    root_path = "/root/path"
    expected_result = "to/file.py"
    assert prepare_file_path(fn, root_path) == expected_result

    # 测试用例2：文件路径不以根路径开头
    fn = "/another/path/to/file.py"
    root_path = "/root/path"
    expected_result = "/another/path/to/file.py"
    assert prepare_file_path(fn, root_path) == expected_result

    # 测试用例3：文件路径与根路径相同
    fn = "/root/path"
    root_path = "/root/path"
    expected_result = ""
    assert prepare_file_path(fn, root_path) == expected_result

    # 测试用例4：根路径为空字符串
    fn = "/path/to/file.py"
    root_path = ""
    expected_result = "path/to/file.py"
    assert prepare_file_path(fn, root_path) == expected_result

    # 测试用例5：文件路径为空字符串
    fn = ""
    root_path = "/root/path"
    expected_result = ""
    assert prepare_file_path(fn, root_path) == expected_result


def test_get_testcase_coverage_data(monkeypatch):
    # 假设你的测试数据目录是 test_data_path
    test_data_path = Path(testdata_dir)  # 请根据实际情况修改
    coverage_db_path = test_data_path / ".coverage"

    # 检查文件是否存在
    assert coverage_db_path.is_file(), f"Coverage db {coverage_db_path} does not exist"

    # 调用方法
    result = get_testcase_coverage_data(["/data/tests/addition_mod"], coverage_db_path)

    # 验证结果
    expected_result = {
        "uttest.test_add.test_add_param": CoverageData(
            name="uttest.test_add.test_add_param",
            files={"/data/tests/addition_mod/add.py": [11, 12]},
        ),
        "uttest.test_add.test_add_dict": CoverageData(
            name="uttest.test_add.test_add_dict",
            files={"/data/tests/addition_mod/add.py": [11, 13, 15, 17, 18, 19, 20, 21]},
        ),
        "uttest.test_add.test_add_2_numbers": CoverageData(
            name="uttest.test_add.test_add_2_numbers",
            files={"/data/tests/addition_mod/add.py": [11, 12]},
        ),
    }
    assert result == expected_result


class TestCoverageFinder:
    def test_find_coverage_db_path_root(self):
        # 测试根目录存在 cov_file
        result = find_coverage_db_path(str(testdata_dir), ".coverage_db")
        assert result == Path(testdata_dir) / ".coverage_db"

        # 测试根目录不存在 cov_file，测试使用 .coverage 文件
        test_dir = Path(testdata_dir).parent
        result = find_coverage_db_path(str(test_dir), "")
        assert result == Path(testdata_dir) / ".coverage"

        test_dir = Path(testdata_dir).parent / "aa"
        result = find_coverage_db_path(str(test_dir), "")
        assert result == Path("")


class TestGenerateCoverageJsonFile:
    @pytest.fixture
    def setup_test_environment(self):
        proj_path = Path(tempfile.TemporaryDirectory().name)
        proj_path.mkdir()

        coverage_file_path = proj_path / "coverage.xml"
        coverage_file_path.touch()

        coverage_json_file = proj_path / "coverage.json"

        cov_file_info = {
            "test_case1": CoverageData(name="test_case1", files={"file1.py": [1, 2, 3]}),
            "test_case2": CoverageData(name="test_case2", files={"file2.py": [4, 5, 6]}),
        }

        return proj_path, coverage_file_path, cov_file_info, coverage_json_file

    def test_generate_coverage_json_file(self, setup_test_environment, monkeypatch):
        (
            proj_path,
            coverage_file_path,
            cov_file_info,
            coverage_json_file,
        ) = setup_test_environment

        # 调用方法
        generate_coverage_json_file(
            proj_path=proj_path,
            coverage_file_path=coverage_file_path,
            cov_file_info=cov_file_info,
            coverage_json_file=coverage_json_file,
        )

        # 验证生成的 JSON 文件是否正确
        with open(coverage_json_file, "r") as f:
            data = json.load(f)

        expected_data = {
            "coverageFile": str(coverage_file_path),
            "coverageType": "cobertura_xml",
            "projectPath": {
                "projectPath": str(proj_path),
                "beforeMove": "",
                "afterMove": "",
            },
            "caseCoverage": [
                {
                    "caseName": "test_case1",
                    "testFiles": [{"fileName": "file1.py", "fileLines": [1, 2, 3]}],
                },
                {
                    "caseName": "test_case2",
                    "testFiles": [{"fileName": "file2.py", "fileLines": [4, 5, 6]}],
                },
            ],
        }

        assert data == expected_data


class TestCollectCoverageReport:
    @pytest.fixture
    def setup_test_environment(self, tmpdir):
        proj_path = Path(tmpdir) / "projtest"
        proj_path.mkdir()

        file_report_path = "report"
        report_path = proj_path / file_report_path
        report_path.mkdir()

        coverage_file_path = report_path / "coverage.xml"
        coverage_file_path.touch()

        coverage_db_path = proj_path / ".coverage"
        coverage_db_path.touch()

        return proj_path, file_report_path, coverage_file_path, coverage_db_path

    def test_collect_coverage_report(self, setup_test_environment, monkeypatch):
        (
            proj_path,
            file_report_path,
            coverage_file_path,
            coverage_db_path,
        ) = setup_test_environment

        # Mock dependencies
        filter_coverage_xml_packages = MagicMock()
        find_coverage_db_path = MagicMock(return_value=coverage_db_path)
        get_testcase_coverage_data = MagicMock(
            return_value={"test_case1": {"name": "test_case1", "files": {"file1.py": [1, 2, 3]}}}
        )
        generate_coverage_json_file = MagicMock()

        monkeypatch.setattr(
            "src.testsolar_pytestx.extend.coverage_extend.filter_coverage_xml_packages",
            filter_coverage_xml_packages,
        )
        monkeypatch.setattr(
            "src.testsolar_pytestx.extend.coverage_extend.find_coverage_db_path",
            find_coverage_db_path,
        )
        monkeypatch.setattr(
            "src.testsolar_pytestx.extend.coverage_extend.get_testcase_coverage_data",
            get_testcase_coverage_data,
        )
        monkeypatch.setattr(
            "src.testsolar_pytestx.extend.coverage_extend.generate_coverage_json_file",
            generate_coverage_json_file,
        )

        # Mock logger
        logger = MagicMock()
        monkeypatch.setattr("src.testsolar_pytestx.extend.coverage_extend.logger", logger)

        # Call the method
        collect_coverage_report(proj_path, file_report_path, ["package1", "package2"])

        # Assert calls
        filter_coverage_xml_packages.assert_called_once_with(
            coverage_file_path, ["package1", "package2"]
        )
        find_coverage_db_path.assert_called_once_with(proj_path, ".coverage")
        get_testcase_coverage_data.assert_called_once_with(
            ["package1", "package2"], coverage_db_path
        )
        generate_coverage_json_file.assert_called_once()

        logger.info.assert_called_with("collect coverage report done")

    def test_collect_coverage_report_no_coverage_file(self, setup_test_environment, monkeypatch):
        proj_path, file_report_path, coverage_file_path, _ = setup_test_environment

        # Remove the coverage.xml file to test the error case
        os.remove(coverage_file_path)

        # Mock logger
        logger = MagicMock()
        monkeypatch.setattr("src.testsolar_pytestx.extend.coverage_extend.logger", logger)

        # Call the method
        collect_coverage_report(proj_path, file_report_path, ["package1", "package2"])

        # Assert that error log is called
        logger.error.assert_called_once_with("File coverage.xml not exist", file=sys.stderr)



import io
import unittest
from pathlib import Path
from typing import Dict, List

from testsolar_testtool_sdk.model.param import EntryParam
from testsolar_testtool_sdk.model.load import LoadResult
from testsolar_testtool_sdk.pipe_reader import read_load_result

from src.testsolar_pytestx.collector import collect_testcases


class CollectorTest(unittest.TestCase):
    testdata_dir: str = str(Path(__file__).parent.parent.absolute().joinpath("testdata"))

    def test_collect_testcases_when_selector_is_valid(self):
        entry = EntryParam(
            TaskId="aa",
            ProjectPath=self.testdata_dir,
            TestSelectors=[
                "test_normal_case.py?test_success",
                "aa/bb/cc/test_in_sub_class.py",
                "test_data_drive.py",
                "errors/test_import_error.py",
                "errors/test_load_error.py",
            ],
            FileReportPath="",
        )

        pipe_io = io.BytesIO()
        collect_testcases(entry, pipe_io)
        pipe_io.seek(0)

        re = read_load_result(pipe_io)

        self.assertEqual(len(re.Tests), 6)
        self.assertEqual(len(re.LoadErrors), 2)

        self.assertEqual(re.Tests[0].Name, "aa/bb/cc/test_in_sub_class.py?TestCompute/test_add")
        self.assertEqual(re.Tests[1].Name, "test_data_drive.py?test_eval/%5B2%2B4-6%5D")
        self.assertEqual(re.Tests[2].Name, "test_data_drive.py?test_eval/%5B3%2B5-8%5D")
        self.assertEqual(re.Tests[3].Name, "test_data_drive.py?test_eval/%5B6%2A9-42%5D")
        self.assertEqual(
            re.Tests[4].Name,
            "test_data_drive.py?test_special_data_drive_name/%5B%E4%B8%AD%E6%96%87-%E5%88%86%E5%8F%B7%2B%5Bid%3A32%5D%5D",
        )

        self.assertEqual(re.Tests[5].Name, "test_normal_case.py?test_success")
        self.assertEqual(re.Tests[5].Attributes["owner"], "foo")
        self.assertEqual(re.Tests[5].Attributes["description"], "测试获取答案")
        self.assertEqual(re.Tests[5].Attributes["tag"], "high")
        self.assertEqual(re.Tests[5].Attributes["extra_attributes"], '[{"env": ["AA", "BB"]}]')

        self.assertEqual(
            re.LoadErrors[0].name,
            "load error of selector: [errors/test_import_error.py]",
        )
        self.assertIn(
            "ModuleNotFoundError: No module named 'bad_import'",
            re.LoadErrors[0].message,
        )
        self.assertEqual(
            re.LoadErrors[1].name, "load error of selector: [errors/test_load_error.py]"
        )
        self.assertIn("SyntaxError: ", re.LoadErrors[1].message)

    def test_collect_testcases_when_select_not_valid(self):
        entry = EntryParam(
            TaskId="aa",
            ProjectPath=self.testdata_dir,
            TestSelectors=[
                "test_data_drive.py",
                "test_not_exist.py",
            ],
            FileReportPath="",
        )

        pipe_io = io.BytesIO()
        collect_testcases(entry, pipe_io)
        pipe_io.seek(0)

        re = read_load_result(pipe_io)

        self.assertEqual(len(re.Tests), 4)
        self.assertEqual(len(re.LoadErrors), 1)
        self.assertIn("test_not_exist.py does not exist, SKIP it", re.LoadErrors[0].message)

    def test_collect_testcases_with_utf8_chars(self):
        entry = EntryParam(
            TaskId="aa",
            ProjectPath=self.testdata_dir,
            TestSelectors=[
                "test_data_drive_zh_cn.py",
            ],
            FileReportPath="",
        )

        pipe_io = io.BytesIO()
        collect_testcases(entry, pipe_io)
        pipe_io.seek(0)

        re = read_load_result(pipe_io)

        self.assertEqual(len(re.Tests), 3)
        self.assertEqual(len(re.LoadErrors), 0)

        self.assertEqual(
            re.Tests[0].Name,
            "test_data_drive_zh_cn.py?test_include/%5B%23%3F-%23%3F%5E%24%25%21/%5D",
        )
        self.assertEqual(
            re.Tests[1].Name,
            "test_data_drive_zh_cn.py?test_include/%5B%E4%B8%AD%E6%96%87-%E4%B8%AD%E6%96%87%E6%B1%89%E5%AD%97%5D",
        )
        self.assertEqual(
            re.Tests[2].Name,
            "test_data_drive_zh_cn.py?test_include/%5B%ED%8C%8C%EC%9D%BC%EC%9D%84%20%EC%B0%BE%EC%9D%84%20%EC%88%98%20%EC%97%86%EC%8A%B5%EB%8B%88%EB%8B%A4-%E3%83%95%E3%82%A1%E3%82%A4%E3%83%AB%E3%81%8C%E8%A6%8B%E3%81%A4%E3%81%8B%E3%82%8A%E3%81%BE%E3%81%9B%E3%82%93%5D",
        )

    def test_collect_testcases_with_case_drive_separator(self):
        entry = EntryParam(
            TaskId="aa",
            ProjectPath=self.testdata_dir,
            TestSelectors=[
                "test_normal_case.py?test_success→压缩机测试",
                "test_normal_case.py?test_success→解压机测试",
                "test_normal_case.py?test_success→循环机测试",
            ],
            FileReportPath="",
        )

        case_records = {}

        def loader_extend(param_1: str, param_2: LoadResult, param_3: Dict[str, List[str]]) -> None:
            case_records.update(param_3)

        pipe_io = io.BytesIO()
        collect_testcases(entry, pipe_io, extra_load_function=loader_extend)
        pipe_io.seek(0)

        re = read_load_result(pipe_io)

        self.assertEqual(len(re.Tests), 1)
        self.assertEqual(len(re.LoadErrors), 0)

        self.assertEqual(re.Tests[0].Name, "test_normal_case.py?test_success")

        self.assertEqual(len(case_records), 1)
        self.assertIn("test_normal_case.py?test_success", case_records)

        records = case_records["test_normal_case.py?test_success"]
        self.assertEqual(len(records), 3)
        self.assertEqual(records[0], "压缩机测试")
        self.assertEqual(records[1], "解压机测试")
        self.assertEqual(records[2], "循环机测试")

    def test_collect_testcases_when_testcase_not_exist(self):
        entry = EntryParam(
            TaskId="aa",
            ProjectPath=self.testdata_dir,
            TestSelectors=[
                "test_normal_case.py?name=not_exist",
            ],
            FileReportPath="",
        )

        pipe_io = io.BytesIO()
        collect_testcases(entry, pipe_io)
        pipe_io.seek(0)

        re = read_load_result(pipe_io)

        self.assertEqual(len(re.LoadErrors), 1)

        self.assertEqual(
            re.LoadErrors[0].name,
            "test_normal_case.py?name=not_exist",
        )

    def test_collect_testcases_with_skipp_error(self):
        entry = EntryParam(
            TaskId="aa",
            ProjectPath=self.testdata_dir,
            TestSelectors=[
                "test_normal_case.py",
                "test_skipped_error.py",
            ],
            FileReportPath="",
        )

        pipe_io = io.BytesIO()
        collect_testcases(entry, pipe_io)
        pipe_io.seek(0)

        re = read_load_result(pipe_io)

        self.assertEqual(len(re.Tests), 3)
        self.assertEqual(len(re.LoadErrors), 1)

    def test_collect_testcases_with_emoji(self):
        entry = EntryParam(
            TaskId="aa",
            ProjectPath=self.testdata_dir,
            TestSelectors=[
                "test_emoji_data_drive.py",
            ],
            FileReportPath="",
        )

        pipe_io = io.BytesIO()
        collect_testcases(entry, pipe_io)
        pipe_io.seek(0)

        re = read_load_result(pipe_io)

        self.assertEqual(len(re.Tests), 1)
        self.assertEqual(len(re.LoadErrors), 0)
        self.assertEqual(
            re.Tests[0].Name,
            "test_emoji_data_drive.py?test_emoji_data_drive_name/%5B%F0%9F%98%84%5D",
        )



import allure


@allure.step
def step_function_with_args(arg1, arg2):
    pass


@allure.step("Step with param '{param}'")
def step_with_placeholder(param):
    pass


@allure.step("Custom step title")
def step_function_with_title():
    pass


def test_step():
    with allure.step("First step"):
        step_with_placeholder("Param value")
        with allure.step("Nested step"):
            step_function_with_args("value1", "value2")
    step_function_with_title()



import pytest


@pytest.mark.parametrize(
    "test_input,expected",
    [
        ("#?", "#?^$%!/"),
        ("中文", "中文汉字"),
        ("파일을 찾을 수 없습니다", "ファイルが見つかりません"),
    ],
)
def test_include(test_input: str, expected: str):
    """
    测试包含UTF-8字符的数据驱动用例
    """
    assert test_input in expected



from dataclasses import dataclass, field, asdict
from typing import List, Dict
import json
import os
import configparser
import sys
import time
import uuid
from xml.dom import minidom
import coverage
from loguru import logger
from pathlib import Path

COVERAGE_DIR: str = "testsolar_coverage"


@dataclass
class TestFileLines:
    """
    数据类，用于存储测试文件名称和行号。
    """

    __test__ = False

    fileName: str
    fileLines: List[int]


@dataclass
class TestCaseCoverage:
    """
    数据类，用于存储测试用例覆盖率。
    """

    __test__ = False

    caseName: str
    testFiles: List[TestFileLines] = field(default_factory=list)


@dataclass
class ProjectPath:
    """
    数据类，用于存储项目路径信息。
    """

    projectPath: str
    beforeMove: str = ""
    afterMove: str = ""


@dataclass
class Coverage:
    """
    数据类，用于存储覆盖率信息。
    """

    coverageFile: str
    coverageType: str
    projectPath: ProjectPath
    caseCoverage: List[TestCaseCoverage] = field(default_factory=list)

    def to_json(self) -> str:
        """
        将 Coverage 对象序列化为 JSON 字符串。

        Returns:
            str: 序列化后的 JSON 字符串。
        """
        return json.dumps(asdict(self), indent=4)


@dataclass
class CoverageData:
    """
    数据类，用于存储单个测试用例的覆盖率数据。
    """

    name: str
    files: Dict[str, List[int]] = field(default_factory=dict)


def check_coverage_enable() -> bool:
    """
    检查是否启用覆盖率。

    Returns:
        bool: 是否启用覆盖率。
    """
    return os.getenv("TESTSOLAR_TTP_ENABLECOVERAGE", "") in ["1", "true"]


def collect_code_packages(testcase_list: List[str]) -> List[str]:
    """
    自动计算被测代码包。

    如果 code_package 为空，则遍历当前目录，自动识别项目中的代码包。
    过滤掉以 '.' 开头的目录、没有 __init__.py 文件的目录以及测试用例目录。

    Args:
        testcase_list (List[str]): 测试用例列表。

    Returns:
        List[str]: 自动识别的代码包列表。
    """
    code_package_env: str = os.environ.get("TESTSOLAR_TTP_COVERAGECODEPACKAGES", "")
    if code_package_env:
        code_package: List[str] = code_package_env.strip().split(";")
    else:
        code_package = []

    # 如果测试用例列表不为空，则过滤掉测试用例目录
    testcase_package_list: List[str] = []
    for it in testcase_list:
        testcase_package_list.append(it.split("/")[0].strip())
    testcase_package_list = list(set(testcase_package_list))

    # 如果没有找到覆盖率包目录，则根据当前目录下的所有目录来筛选
    if not code_package:
        for it in os.listdir(os.getcwd()):
            if not os.path.isdir(it):
                continue
            if not os.path.exists(Path(it) / "__init__.py"):
                continue
            if it[0] == ".":
                continue
            if testcase_list and it in testcase_package_list:
                continue
            code_package.append(it)

    return code_package


def filter_coverage_xml_packages(xml_path: Path, code_package: List[str]) -> None:
    """
    过滤 coverage.xml 文件中非测试覆盖包目录。

    Args:
        xml_path (str): coverage.xml 文件路径。
        code_package (List[str]): 被测代码包列表。
    """
    start_time: float = time.time()
    logger.info(f"code_package: {code_package}")
    with open(xml_path, "r") as fp:
        dom = minidom.parse(fp)
        root = dom.documentElement
        source = root.getElementsByTagName("source")[0]
        packages = root.getElementsByTagName("package")
        for package in packages:
            name = package.getAttribute("name")
            if len(code_package) > 0:
                for source in code_package:
                    if name == source or name.startswith(source + "."):
                        break
                else:
                    logger.info("Package %s ignored" % name)
                    package.parentNode.removeChild(package)

        with open(xml_path, "w") as fd:
            dom.writexml(fd)
    logger.info("filter_coverage_xml_packages cost time: %s" % (time.time() - start_time))


def convert_coverage_data(
    result: Dict[str, CoverageData],
    rav_fn: str,
    line_map: List[int],
    context_map: Dict[int, List[str]],
) -> Dict[str, CoverageData]:
    """
    转换覆盖率数据。

    Args:
        result (Dict[str, CoverageData]): 当前的覆盖率数据字典。
        rav_fn (str): 文件路径。
        line_map (List[int]): 覆盖的行号列表。
        context_map (Dict[int, List[str]]): 行号与测试用例的映射。

    Returns:
        Dict[str, CoverageData]: 更新后的覆盖率数据字典。
    """
    for line in line_map:
        test_case_list = context_map.get(line, [])
        for test_case in test_case_list:
            if not test_case:
                continue
            try:
                name = test_case[: test_case.index("|")]
            except ValueError:
                name = test_case
            items = name.split("::")
            if items[0].endswith(".py"):
                items[0] = items[0][:-3].replace(os.sep, ".")
            name = ".".join(items)
            if name not in result:
                result[name] = CoverageData(name=name)
            if rav_fn not in result[name].files:
                result[name].files[rav_fn] = []
            result[name].files[rav_fn].append(line)
    return result


def get_testcase_coverage_data(
    code_package: List[str], coverage_db_path: Path
) -> Dict[str, CoverageData]:
    """
    获取测试用例的覆盖率数据。

    Args:
        code_package (List[str]): 被测代码包列表。
        coverage_db_path (str): 覆盖率数据库文件路径。

    Returns:
        Dict[str, CoverageData]: 包含覆盖率数据的字典。
    """
    if not os.path.isfile(coverage_db_path):
        raise RuntimeError(f"Coverage db {coverage_db_path} not exist")
    root_path: str = os.path.dirname(os.path.abspath(coverage_db_path))
    result: Dict[str, CoverageData] = {}

    # 加载覆盖率数据并获取计量的文件
    cov = coverage.Coverage(data_file=coverage_db_path)
    cov.load()
    file_set = cov.get_data().measured_files()
    logger.info("Coverage data loaded")

    for fn in file_set:
        rav_fn = prepare_file_path(fn, root_path)
        if not is_file_in_code_package(rav_fn, code_package):
            continue

        line_map = cov.get_data().lines(fn)
        context_map = cov.get_data().contexts_by_lineno(fn)
        if line_map is not None:
            try:
                result = convert_coverage_data(result, rav_fn, line_map, context_map)
            except Exception as e:
                logger.error(f"Error processing file {fn}: {e}")

    return result


def prepare_file_path(fn: str, root_path: str) -> str:
    """
    准备文件路径。

    Args:
        fn (str): 文件路径。
        root_path (str): 根路径。

    Returns:
        str: 处理后的文件路径。
    """
    if fn.startswith(root_path):
        fn = fn[len(root_path) + 1 :]
    return fn


def is_file_in_code_package(fn: str, code_package: List[str]) -> bool:
    """
    检查文件是否在代码包中。

    Args:
        fn (str): 文件路径。
        code_package (List[str]): 代码包列表。

    Returns:
        bool: 如果文件在代码包中，则返回 True；否则返回 False。
    """
    for source in code_package:
        if fn.startswith(source):
            return True
    return False


def find_coverage_db_path(proj: str, cov_file: str) -> Path:
    """
    查找覆盖率数据库文件路径。

    Args:
        proj (str): 项目路径。
        cov_file (str): 覆盖率文件名。

    Returns:
        Path: 覆盖率数据库文件路径。如果未找到，返回空路径。
    """
    # 检查是否根目录存在db文件
    coverage_db_path = Path(proj) / cov_file
    if coverage_db_path.is_file():
        logger.info("Found coverage db file: {}", coverage_db_path)
        return coverage_db_path

    # 查找 .coveragerc 文件 data_file 配置
    coveragerc_file_path = Path(proj) / ".coveragerc"
    if coveragerc_file_path.is_file():
        config = configparser.ConfigParser()
        config.read(coveragerc_file_path)

        # 检查 [run] 部分中的 data_file 配置项
        if "run" in config and "data_file" in config["run"]:
            coverage_path = Path(config["run"]["data_file"].strip())
            if coverage_path.is_file():
                logger.info("Found coverage db file: {}", coverage_path)
                return coverage_path

    # 遍历项目目录，查找 .coverage 文件
    for root, _, files in os.walk(proj):
        for f_name in files:
            if f_name == ".coverage":
                coverage_path = Path(root) / f_name
                logger.info("Found coverage db file: {}", coverage_path)
                return coverage_path

    # 如果未找到覆盖率数据库文件，返回空路径
    return Path("")


def generate_coverage_json_file(
    proj_path: str,
    coverage_file_path: Path,
    cov_file_info: Dict[str, CoverageData],
    coverage_json_file: Path,
) -> None:
    """
    生成覆盖率 JSON 文件。

    Args:
        proj_path (str): 项目路径。
        coverage_file_path (str): 覆盖率文件路径。
        cov_file_info (Dict[str, CoverageData]): 测试用例覆盖率信息。
        coverage_json_file (str): 覆盖率 JSON 文件保存路径。
    """
    # 创建 ProjectPath 对象，存储项目路径信息
    project_path = ProjectPath(
        projectPath=str(proj_path),
        beforeMove="",
        afterMove="",  # 确保转换为字符串
    )

    # 创建 Coverage 对象，存储覆盖率信息
    coverage_data = Coverage(
        coverageFile=str(coverage_file_path),
        coverageType="cobertura_xml",
        projectPath=project_path,
    )

    # 填充测试用例覆盖率文件行信息
    if cov_file_info:
        for case_name, file_covs in cov_file_info.items():
            test_files = [
                TestFileLines(fileName=file_name, fileLines=file_lines)
                for file_name, file_lines in file_covs.files.items()
            ]
            test_case_coverage = TestCaseCoverage(caseName=case_name, testFiles=test_files)
            coverage_data.caseCoverage.append(test_case_coverage)
    else:
        logger.warning("No test case coverage data found")
        coverage_data.caseCoverage = []

    # 将 Coverage 对象序列化为 JSON 并写入文件
    with open(coverage_json_file, "w") as f:
        f.write(coverage_data.to_json())

    logger.info(f"Coverage data saved to {coverage_json_file}")


def collect_coverage_report(proj_path: str, file_report_path: str, code_package: List[str]) -> None:
    """
    处理覆盖率并生成覆盖率报告。

    Args:
        proj_path (str): 项目路径。
        code_package (List[str]): 被测代码包列表。
    """
    # 定义覆盖率文件路径和 JSON 文件路径
    coverage_file_path: Path = Path(proj_path) / file_report_path / "coverage.xml"
    coverage_json_dir = Path(proj_path) / "testsolar_coverage"
    if not coverage_json_dir.exists():
        coverage_json_dir.mkdir()
    unique_string = str(uuid.uuid4())
    coverage_json_file: Path = coverage_json_dir / f"{unique_string}.json"

    # 检查覆盖率文件是否存在
    if not os.path.exists(coverage_file_path):
        logger.error("File coverage.xml not exist", file=sys.stderr)
        return

    # 过滤 coverage.xml 文件中的包信息
    logger.info("filter coverage.xml packages")
    filter_coverage_xml_packages(coverage_file_path, code_package)

    # 查找覆盖率数据库文件路径
    coverage_db_path = find_coverage_db_path(proj_path, ".coverage")

    # 获取测试用例的覆盖率数据
    if coverage_db_path:
        cov_file_info = get_testcase_coverage_data(code_package, coverage_db_path)
    else:
        # 不存在覆盖率数据库文件，则不生成覆盖率文件行信息
        cov_file_info = {}

    # 生成覆盖率 JSON 文件
    generate_coverage_json_file(proj_path, coverage_file_path, cov_file_info, coverage_json_file)

    logger.info("collect coverage report done")



import io
import sys
import pytest
import contextlib
from typing import List, Tuple, TextIO, TypeVar

T = TypeVar("T")

MAX_CHAR_SIZE = int(100 * 1024 * 1024 / 8 / 4)  # 100MB 对应的UTF-8编码字符数


def exceeds_max_size(stream: TextIO) -> bool:
    # 保存当前流的位置
    current_position = stream.tell()

    # 移动到流的末尾，获取流的总字符数
    stream.seek(0, 2)  # 2 表示从流的末尾开始计算偏移量
    total_size = stream.tell()

    # 恢复流的位置
    stream.seek(current_position)

    return total_size > MAX_CHAR_SIZE


class TeeStream:
    def __init__(self, *streams: TextIO) -> None:
        self.streams = streams

    def write(self, data: str) -> None:
        for stream in self.streams:
            # 如果是标准输出流/标准错误流则直接写入
            if not isinstance(stream, io.StringIO):
                stream.write(data)
            # 若超过最大字符数则不写入，避免内存占用过高
            elif not exceeds_max_size(stream=stream):
                stream.write(data)

    def flush(self) -> None:
        for stream in self.streams:
            stream.flush()

    def isatty(self) -> bool:
        return any(stream.isatty() for stream in self.streams)


def pytest_main_with_output(args: List[str], plugin: T) -> Tuple[str, str, int]:
    exit_code = 0
    stdout_capture = io.StringIO()
    stderr_capture = io.StringIO()
    stdout_stream = TeeStream(sys.stdout, stdout_capture)
    stderr_stream = TeeStream(sys.stderr, stderr_capture)
    with contextlib.redirect_stdout(stdout_stream), contextlib.redirect_stderr(stderr_stream):  # type: ignore
        exit_code = pytest.main(args, plugins=[plugin])
    captured_stdout = stdout_capture.getvalue()
    captured_stderr = stderr_capture.getvalue()
    return captured_stdout, captured_stderr, int(exit_code)



import json
import os
import shutil
from dataclasses import dataclass, field
from datetime import datetime
from typing import List, Dict, Optional
from loguru import logger
from pathlib import Path

from dacite import from_dict
from testsolar_testtool_sdk.model.testresult import (
    TestCaseLog,
    LogLevel,
    TestResult,
    ResultType,
    TestCaseStep,
)


# 定义数据类，用于表示Allure报告中的各种结构
@dataclass
class StatusDetails:
    message: Optional[str] = None
    trace: Optional[str] = None


@dataclass
class Parameter:
    name: str
    value: str


@dataclass
class Attachments:
    name: str
    source: str
    type: str


@dataclass
class Step:
    name: str
    status: str
    start: int
    stop: int
    parameters: Optional[List[Parameter]] = field(default_factory=list)  # type: ignore
    steps: Optional[List["Step"]] = field(default_factory=list)  # type: ignore
    statusDetails: Optional[StatusDetails] = None
    attachments: Optional[List[Attachments]] = None


@dataclass
class AllureData:
    name: str
    status: str
    start: int
    stop: int
    uuid: str
    historyId: str
    testCaseId: str
    fullName: str
    steps: List[Step] = field(default_factory=list)
    labels: List[Dict[str, str]] = field(default_factory=list)
    attachments: Optional[List[Attachments]] = None


def check_allure_enable() -> bool:
    """
    检查环境变量以确定是否启用Allure报告。
    """
    return os.getenv("TESTSOLAR_TTP_ENABLEALLURE", "") in ["1", "true"]


def initialization_allure_dir(allure_dir: str) -> None:
    """
    初始化 Allure 报告目录。如果指定的目录存在，则删除该目录及其所有内容。然后重新创建一个空目录。

    :param allure_dir: Allure 报告目录路径
    """
    logger.info(f"Initializing Allure directory: {allure_dir}")
    if Path(allure_dir).exists():
        logger.info(f"Directory {allure_dir} exists. Removing it.")
        shutil.rmtree(allure_dir)
    os.makedirs(allure_dir, exist_ok=True)
    logger.info(f"Directory {allure_dir} created.")


def generate_allure_results(
    test_data: Dict[str, TestResult], file_name: str, attachment_dir: str
) -> None:
    """
    生成 Allure 报告结果。

    :param test_data: 测试数据字典
    :param file_name: 包含 Allure 报告的 JSON 文件的名称
    :param attachment_dir: 附件目录
    """
    logger.info("Start to generate allure results")
    logger.debug(f"Reading Allure data from file: {file_name}")
    with open(file_name) as fp:
        allure_data = from_dict(data_class=AllureData, data=json.loads(fp.read()))
        full_name = allure_data.fullName.replace("#", ".")
        logger.debug(f"Parsed Allure data for test case: {full_name}")

        for testcase_name in test_data.keys():
            logger.info(f"Processing test case: {testcase_name}")
            step_info: List[TestCaseStep] = []
            testcase_format_name = ".".join(testcase_name.replace(".py?", os.sep).split(os.sep))
            logger.debug(f"Formatted test case name: {testcase_format_name}")

            if full_name != testcase_format_name:
                logger.info(
                    f"Test case name {full_name} does not match {testcase_format_name}. Skipping."
                )
                continue

            if allure_data.steps:
                logger.info(f"Generating step info for test case {full_name}")
                step_info = gen_allure_step_info(allure_data.steps, attachment_dir)

            if allure_data.attachments:
                logger.info(f"Processing attachments for test case {full_name}")
                for attachment in allure_data.attachments:
                    attachment_path = Path(attachment_dir).joinpath(attachment.source)
                    logger.debug(f"Attachment path: {str(attachment_path)}")

                    if Path(attachment_path).is_file():
                        logger.info(f"Reading attachment: {attachment_path}")
                        with open(attachment_path, "r", encoding="utf-8", errors="ignore") as f:
                            log_content = f.read()
                            log_info = TestCaseLog(
                                Time=format_allure_time(allure_data.start),
                                Level=LogLevel.INFO,
                                Content=f"Attachment {attachment.name}:\n{log_content}",
                            )
                            step_info.append(
                                TestCaseStep(
                                    Title="Testcase Stdout:",
                                    Logs=[log_info],
                                    StartTime=format_allure_time(allure_data.start),
                                    EndTime=format_allure_time(allure_data.stop),
                                    ResultType=ResultType.SUCCEED
                                    if allure_data.status == "passed"
                                    else ResultType.FAILED,
                                )
                            )
            test_data[testcase_name].Steps.clear()
            test_data[testcase_name].Steps.extend(step_info)
            logger.info(f"Finished processing test case: {testcase_name}")


def format_allure_time(timestamp: float) -> datetime:
    """
    格式化 Allure 时间戳。

    :param timestamp: 时间戳
    :return: 格式化的日期时间对象
    """
    formatted_time = datetime.fromtimestamp(timestamp / 1000)
    logger.debug(f"Formatted timestamp {timestamp} to {formatted_time}")
    return formatted_time


def gen_allure_step_info(
    steps: List[Step], attachment_dir: str, index: int = 0
) -> List[TestCaseStep]:
    """
    生成 Allure 步骤信息。

    :param steps: 步骤列表
    :param attachment_dir: 附件目录
    :param index: 步骤索引
    :return: 测试用例步骤列表
    """
    logger.info("Generating allure step info")
    case_steps = []
    for step in steps:
        index += 1
        result = step.status
        result_type: ResultType
        if result == "passed":
            result_type = ResultType.SUCCEED
        elif result == "skipped":
            result_type = ResultType.IGNORED
        else:
            result_type = ResultType.FAILED

        log = "\n"
        if step.parameters:
            for param in step.parameters:
                log += "%-30s%-20s\n" % (
                    "key: {}".format(param.name),
                    "value: {}".format(param.value),
                )
        if step.statusDetails:
            if step.statusDetails.message and step.statusDetails.trace:
                log += step.statusDetails.message + step.statusDetails.trace
        if step.attachments:
            for attachment in step.attachments:
                attachment_path = Path(attachment_dir).joinpath(attachment.source)
                logger.debug(f"Attachment path: {str(attachment_path)}")

                if Path(attachment_path).exists():
                    logger.info(f"Reading attachment: {attachment_path}")
                    with open(attachment_path, "r") as f:
                        log += f"\n{attachment.name}:\n" + f.read() + "\n\n"

        log_info = TestCaseLog(
            Time=format_allure_time(step.start),
            Level=LogLevel.ERROR if result == "failed" else LogLevel.INFO,
            Content=log,
        )
        step_info = TestCaseStep(
            Title="{}: {}".format(".".join(list(str(index))), step.name),
            Logs=[log_info],
            StartTime=format_allure_time(step.start),
            EndTime=format_allure_time(step.stop),
            ResultType=result_type,
        )

        logger.info(f"Generated step info: {step_info}")
        case_steps.append(step_info)
        if step.steps:
            case_steps.extend(gen_allure_step_info(step.steps, attachment_dir, index * 10))
    return case_steps



import io
import os
from datetime import datetime
from pathlib import Path
from unittest import TestCase

import pytest
from testsolar_testtool_sdk.model.test import TestCase as TestSolar_TestCase
from testsolar_testtool_sdk.model.testresult import (
    TestResult,
)
from testsolar_testtool_sdk.pipe_reader import read_test_result

from run import run_testcases_from_args
from testsolar_pytestx.extend.allure_extend import (
    generate_allure_results,
    gen_allure_step_info,
    format_allure_time,
    ResultType,
    Step,
    Attachments,
)


@pytest.fixture
def test_data():
    return {
        "test_module.test_case_1": TestResult(
            Test=TestSolar_TestCase(Name="test_module.test_case_1", Attributes={}),
            StartTime=datetime.now(),
            ResultType=ResultType.SUCCEED,
            Message="",
            Steps=[],
        ),
        "test_module.test_case_2": TestResult(
            Test=TestSolar_TestCase(Name="test_module.test_case_2", Attributes={}),
            StartTime=datetime.now(),
            ResultType=ResultType.SUCCEED,
            Message="",
            Steps=[],
        ),
    }


class TestAllureResults:
    def test_generate_allure_results(self, test_data):
        test_attachments_path = (
            Path(__file__).parent.parent.absolute().joinpath("testdata/allure_attachments")
        )
        result_file_path = Path(test_attachments_path).joinpath("results.json")
        generate_allure_results(test_data, result_file_path, test_attachments_path)

        assert len(test_data["test_module.test_case_1"].Steps) == 4
        assert test_data["test_module.test_case_1"].Steps[0].Title == "1: step1"
        assert test_data["test_module.test_case_1"].Steps[0].ResultType == ResultType.SUCCEED
        # Check the content of the attachment
        assert (
            "This is the content of 创建仓库 attachment."
            in test_data["test_module.test_case_1"].Steps[0].Logs[0].Content
        )
        assert (
            "This is the content of stdout attachment."
            in test_data["test_module.test_case_1"].Steps[-1].Logs[0].Content
        )

    def test_gen_allure_step_info(self):
        steps = [
            Step(
                name="step1",
                status="passed",
                start=1622547800000,
                stop=1622547810000,
                parameters=[],
                steps=[],
                statusDetails=None,
                attachments=[
                    Attachments(
                        name="example attachment",
                        source="example-attachment.txt",
                        type="text/plain",
                    )
                ],
            )
        ]
        result = gen_allure_step_info(steps, "")
        assert len(result) == 1
        assert result[0].Title == "1: step1"
        assert result[0].ResultType == ResultType.SUCCEED

    def test_format_allure_time(self):
        timestamp = 1622547800000
        result = format_allure_time(timestamp)
        assert result == datetime.fromtimestamp(timestamp / 1000)


class TestExecuteEntry(TestCase):
    testdata_dir: str = str(Path(__file__).parent.parent.absolute().joinpath("testdata"))

    def test_run_testcases_from_args(self):
        os.environ["TESTSOLAR_TTP_ENABLEALLURE"] = "1"
        pipe_io = io.BytesIO()
        run_testcases_from_args(
            args=[
                "run.py",
                Path.joinpath(Path(self.testdata_dir), "allure_entry.json"),
            ],
            workspace=self.testdata_dir,
            pipe_io=pipe_io,
        )

        # testcase running
        pipe_io.seek(0)
        start = read_test_result(pipe_io)
        self.assertEqual(start.ResultType, ResultType.RUNNING)
        self.assertEqual(start.Test.Name, "allure/allure_step_test.py?test_step")

        # testcase finish
        stop = read_test_result(pipe_io)
        self.assertEqual(stop.ResultType, ResultType.SUCCEED)
        self.assertEqual(stop.Test.Name, "allure/allure_step_test.py?test_step")
        self.assertEqual(stop.Message, "")
        self.assertEqual(type(stop.Steps), list)
        self.assertEqual(len(stop.Steps), 5)



import os
import shlex
from typing import List
from loguru import logger
from pathlib import Path

from .extend.coverage_extend import check_coverage_enable, collect_code_packages


def append_coverage_args(
    args: List[str], valid_selectors: List[str], file_report_path: str
) -> List[str]:
    """
    将覆盖率参数添加到命令行数组中，并返回被测代码包列表

    Args:
        args (List[str]): 命令行参数列表
        valid_selectors (List[str]): 有效的用例列表
        file_report_path (str): 文件报告路径

    Returns:
        List[str]: 被测代码包列表
    """
    enable_coverage: bool = check_coverage_enable()
    code_package: List[str] = []
    coverage_file_path = Path(file_report_path) / "coverage.xml"

    if enable_coverage:
        # 自动计算和识别项目中的代码包
        code_package = collect_code_packages(valid_selectors)
        if code_package:
            # 如果找到了代码包，添加覆盖率相关的参数
            args.extend(
                [
                    "--cov=.",  # 指定要测量覆盖率的目录为当前目录
                    f"--cov-report=xml:{coverage_file_path}",  # 生成 XML 格式的覆盖率报告，并保存到 此进程唯一的目录中
                    "--cov-context=test",  # 在覆盖率报告中添加测试用例名称作为上下文信息
                ]
            )
        else:
            # 如果没有找到代码包，记录警告日志，表示不会收集覆盖率数据
            logger.warning("No source files found, coverage will not be collected")

    return code_package


def append_extra_args(args: List[str]) -> None:
    """
    将用户配置的额外参数作为命令行数组传递给pytest

    注意用户配置的字符串中可能存在空格类型参数，比如 -m "not m3"，因此需要使用shlex.split来分割参数
    """
    extra_args = os.environ.get("TESTSOLAR_TTP_EXTRAARGS", "")
    if extra_args:
        args.extend(shlex.split(extra_args))



import io
import unittest
from datetime import datetime, timedelta
from pathlib import Path
from unittest import mock

from testsolar_testtool_sdk.model.param import EntryParam
from testsolar_testtool_sdk.model.testresult import ResultType, LogLevel
from testsolar_testtool_sdk.pipe_reader import read_test_result

from src.testsolar_pytestx.executor import run_testcases, append_extra_args


def convert_to_datetime(raw: str) -> datetime:
    dt = datetime.strptime(raw, "%Y-%m-%dT%H:%M:%S.%fZ")
    return dt


class ExecutorTest(unittest.TestCase):
    testdata_dir = Path(__file__).parent.parent.absolute().joinpath("testdata")

    def test_run_success_testcase_with_logs(self):
        entry = EntryParam(
            TaskId="aa",
            ProjectPath=str(self.testdata_dir),
            TestSelectors=[
                "test_normal_case.py?name=test_success&tag=A&priority=High",
            ],
            FileReportPath="",
        )

        current_time = datetime.utcnow()

        pipe_io = io.BytesIO()
        run_testcases(entry, pipe_io)
        pipe_io.seek(0)

        start = read_test_result(pipe_io)
        self.assertEqual(start.ResultType, ResultType.RUNNING)

        end = read_test_result(pipe_io)
        self.assertEqual(end.Test.Name, "test_normal_case.py?test_success")
        self.assertEqual(end.Test.Attributes["tag"], "high")
        self.assertEqual(end.Test.Attributes["owner"], "foo")
        elapse: timedelta = convert_to_datetime(str(end.StartTime)) - current_time
        self.assertLess(elapse.total_seconds(), 0.2)
        elapse_end: timedelta = convert_to_datetime(str(end.EndTime)) - current_time
        self.assertLess(elapse_end.total_seconds(), 0.2)
        self.assertGreater(elapse_end.total_seconds(), 0)
        self.assertEqual(end.ResultType, ResultType.SUCCEED)

        self.assertEqual(len(end.Steps), 3)

        # 检查Setup的时间是否符合要求
        step1 = end.Steps[0]
        self.assertEqual(step1.Title, "Setup")
        elapse = convert_to_datetime(str(step1.StartTime)) - current_time
        self.assertLess(elapse.total_seconds(), 0.2)
        self.assertGreater(elapse.total_seconds(), 0)
        elapse = convert_to_datetime(str(step1.EndTime)) - current_time
        self.assertLess(elapse.total_seconds(), 0.2)
        self.assertGreater(elapse.total_seconds(), 0)

        # 检查Log的时间是否符合要求
        self.assertEqual(len(step1.Logs), 1)
        self.assertEqual(step1.ResultType, ResultType.SUCCEED)
        log = step1.Logs[0]
        self.assertEqual(log.Level, LogLevel.INFO)
        self.assertIn("this is setup", log.Content)
        elapse = convert_to_datetime(str(log.Time)) - current_time
        self.assertGreater(elapse.total_seconds(), 0)
        self.assertLess(elapse.total_seconds(), 0.2)

        # 检查 Run TestCase 时间是否符合要求
        step2 = end.Steps[1]
        self.assertEqual(step2.Title, "Run TestCase")
        elapse = convert_to_datetime(str(step2.StartTime)) - current_time
        self.assertLess(elapse.total_seconds(), 0.2)
        self.assertGreater(elapse.total_seconds(), 0)
        elapse = convert_to_datetime(str(step2.EndTime)) - current_time
        self.assertLess(elapse.total_seconds(), 0.2)
        self.assertGreater(elapse.total_seconds(), 0)

        self.assertEqual(len(step2.Logs), 1)
        self.assertEqual(step2.Logs[0].Level, LogLevel.INFO)
        self.assertEqual(step2.ResultType, ResultType.SUCCEED)
        self.assertIn("this is print sample output", step2.Logs[0].Content)

        # 检查 Teardown 是否是否符合要求
        step3 = end.Steps[2]
        self.assertEqual(step3.Title, "Teardown")
        self.assertEqual(len(step3.Logs), 1)
        self.assertEqual(step3.Logs[0].Level, LogLevel.INFO)
        self.assertEqual(step3.ResultType, ResultType.SUCCEED)
        self.assertEqual(
            step3.Logs[0].Content,
            """this is setup
this is print sample output
this is teardown
""",
        )
        elapse = convert_to_datetime(str(step3.StartTime)) - current_time
        self.assertLess(elapse.total_seconds(), 0.2)
        self.assertGreater(elapse.total_seconds(), 0)
        elapse = convert_to_datetime(str(step3.EndTime)) - current_time
        self.assertLess(elapse.total_seconds(), 0.2)
        self.assertGreater(elapse.total_seconds(), 0)

    def test_run_success_testcase_with_one_invalid_selector(self):
        entry = EntryParam(
            TaskId="aa",
            ProjectPath=str(self.testdata_dir),
            TestSelectors=[
                "test_normal_case.py?name=test_success",
                "test_invalid_case.py?test_success",
            ],
            FileReportPath="",
        )

        pipe_io = io.BytesIO()
        run_testcases(entry, pipe_io)
        pipe_io.seek(0)

        start = read_test_result(pipe_io)
        self.assertEqual(start.ResultType, ResultType.RUNNING)

    def test_run_failed_testcase_with_log(self):
        entry = EntryParam(
            TaskId="aa",
            ProjectPath=str(self.testdata_dir),
            TestSelectors=[
                "test_normal_case.py?test_failed&priority=High",
            ],
            FileReportPath="",
        )

        pipe_io = io.BytesIO()
        run_testcases(entry, pipe_io)
        pipe_io.seek(0)

        start = read_test_result(pipe_io)
        self.assertEqual(start.ResultType, ResultType.RUNNING)

        end = read_test_result(pipe_io)
        self.assertEqual(end.ResultType, ResultType.FAILED)
        self.assertEqual(len(end.Steps), 3)
        self.assertIn("testdata/test_normal_case.py", end.Message)

        step2 = end.Steps[1]
        self.assertEqual(len(step2.Logs), 1)
        self.assertEqual(step2.Logs[0].Level, LogLevel.ERROR)
        self.assertEqual(step2.ResultType, ResultType.FAILED)
        self.assertIn("E       assert 4 == 6", step2.Logs[0].Content)

    def test_run_failed_testcase_with_raise_error(self):
        entry = EntryParam(
            TaskId="aa",
            ProjectPath=str(self.testdata_dir),
            TestSelectors=[
                "test_normal_case.py?test_raise_error",
            ],
            FileReportPath="",
        )

        pipe_io = io.BytesIO()
        run_testcases(entry, pipe_io)
        pipe_io.seek(0)

        start = read_test_result(pipe_io)
        self.assertEqual(start.ResultType, ResultType.RUNNING)

        end = read_test_result(pipe_io)
        self.assertEqual(end.ResultType, ResultType.FAILED)
        self.assertEqual(len(end.Steps), 3)

        step2 = end.Steps[1]
        self.assertEqual(len(step2.Logs), 1)
        self.assertEqual(step2.Logs[0].Level, LogLevel.ERROR)
        self.assertEqual(step2.ResultType, ResultType.FAILED)
        self.assertIn("E       RuntimeError: this is raise runtime error", step2.Logs[0].Content)

    def test_run_skipped_testcase(self):
        entry = EntryParam(
            TaskId="aa",
            ProjectPath=str(self.testdata_dir),
            TestSelectors=[
                "test_skipped.py?test_filtered",
            ],
            FileReportPath="",
        )

        pipe_io = io.BytesIO()
        run_testcases(entry, pipe_io)
        pipe_io.seek(0)

        start = read_test_result(pipe_io)
        self.assertEqual(start.ResultType, ResultType.RUNNING)

        end = read_test_result(pipe_io)
        self.assertEqual(end.ResultType, ResultType.IGNORED)
        self.assertEqual(len(end.Steps), 2)
        self.assertEqual(end.Message, "Skipped: no way of currently testing this")

    def test_run_datadrive_with_single_value(self):
        entry = EntryParam(
            TaskId="aa",
            ProjectPath=str(self.testdata_dir),
            TestSelectors=[
                "test_data_drive.py?test_eval/[2+4-6]",
            ],
            FileReportPath="",
        )

        pipe_io = io.BytesIO()
        run_testcases(entry, pipe_io)
        pipe_io.seek(0)

        start = read_test_result(pipe_io)
        self.assertEqual(start.ResultType, ResultType.RUNNING)

        end = read_test_result(pipe_io)
        self.assertEqual(end.ResultType, ResultType.SUCCEED)
        self.assertEqual(len(end.Steps), 3)

    def test_run_datadrive_with_utf8_str(self):
        entry = EntryParam(
            TaskId="aa",
            ProjectPath=str(self.testdata_dir),
            TestSelectors=[
                "test_data_drive_zh_cn.py?test_include/[中文-中文汉字]",
            ],
            FileReportPath="",
        )

        pipe_io = io.BytesIO()
        run_testcases(entry, pipe_io)
        pipe_io.seek(0)

        start = read_test_result(pipe_io)
        self.assertEqual(start.ResultType, ResultType.RUNNING)

        end = read_test_result(pipe_io)
        self.assertEqual(end.ResultType, ResultType.SUCCEED)
        self.assertEqual(len(end.Steps), 3)

    def test_split_args_with_space(self):
        args = []

        with mock.patch.dict(
            "os.environ",
            {"TESTSOLAR_TTP_EXTRAARGS": '-m "not p3" -t timeout -l "fast iu897 nuh"'},
            clear=True,
        ):
            append_extra_args(args)

            self.assertEqual(len(args), 6)
            self.assertEqual(args[0], "-m")
            self.assertEqual(args[1], "not p3")
            self.assertEqual(args[2], "-t")
            self.assertEqual(args[3], "timeout")
            self.assertEqual(args[4], "-l")
            self.assertEqual(args[5], "fast iu897 nuh")

    def test_run_not_exist_selector(self):
        entry = EntryParam(
            TaskId="aa",
            ProjectPath=str(self.testdata_dir),
            TestSelectors=[
                "test_normal_case.py?name=not_exist",
            ],
            FileReportPath="",
        )

        pipe_io = io.BytesIO()
        run_testcases(entry, pipe_io)
        pipe_io.seek(0)

        result = read_test_result(pipe_io)
        self.assertEqual(result.ResultType, ResultType.FAILED)
        self.assertEqual(result.Test.Name, "test_normal_case.py?name=not_exist")

    def test_run_testcase_with_emoji_data_drive(self):
        entry = EntryParam(
            TaskId="aa",
            ProjectPath=str(self.testdata_dir),
            TestSelectors=[
                "test_emoji_data_drive.py?test_emoji_data_drive_name/[\U0001f604]",
            ],
            FileReportPath="",
        )

        pipe_io = io.BytesIO()
        run_testcases(entry, pipe_io)
        pipe_io.seek(0)

        start = read_test_result(pipe_io)
        self.assertEqual(
            start.Test.Name, "test_emoji_data_drive.py?test_emoji_data_drive_name/[\U0001f604]"
        )
        self.assertEqual(start.ResultType, ResultType.RUNNING)



import pytest


@pytest.mark.parametrize("test_input,expected", [("3+5", 8), ("2+4", 6), ("6*9", 42)])
def test_eval(test_input, expected):
    assert eval(test_input) == expected


@pytest.mark.parametrize("test_input", ["中文-分号+[id:32]"])
def test_special_data_drive_name(test_input):
    assert "bad" not in test_input
